<?php
/**
 * Since 2015 Ewonta
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 wesite only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 *  @author     Ewonta <support@ewonta.ru>
 *  @copyright  Since 2015 Ewonta
 *  @license    Valid for 1 website (or project) for each purchase of license
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Ewcryptocloud extends PaymentModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'ewcryptocloud';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'Ewonta';
        $this->need_instance = 0;
        $this->controllers = array('payment', 'validation', 'orderstate');
        $this->currencies      = true;
		$this->currencies_mode = 'checkbox';

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('CryptoCloud');
        $this->description = $this->l('Accepting Payments via - CryptoCloud');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false)
        {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        Configuration::updateValue('EWCRYPTOCLOUD_API_KEY', '');
        Configuration::updateValue('EWCRYPTOCLOUD_SHOP_ID', '');
        Configuration::updateValue('EWCRYPTOCLOUD_SECRET_KEY', '');
        Configuration::updateValue('EWCRYPTOCLOUD_ID_STATE_AFTER_CREATE', 3);
        Configuration::updateValue('EWCRYPTOCLOUD_ID_STATE_SUCCESS', 2);
        Configuration::updateValue('EWCRYPTOCLOUD_ID_STATE_FAIL', 8);

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayPaymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('EWCRYPTOCLOUD_API_KEY');
        Configuration::deleteByName('EWCRYPTOCLOUD_SHOP_ID');
        Configuration::deleteByName('EWCRYPTOCLOUD_ID_STATE_AFTER_CREATE');
        Configuration::deleteByName('EWCRYPTOCLOUD_ID_STATE_SUCCESS');
        Configuration::deleteByName('EWCRYPTOCLOUD_ID_STATE_FAIL');
        Configuration::deleteByName('EWCRYPTOCLOUD_SECRET_KEY');
        
        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitEwcryptocloudModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign(
            array(
                'callback_link' => $this->context->link->getModuleLink($this->name, 'callback'),
                'module_dir' => $this->_path,
            )
        );

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEwcryptocloudModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l('To get an ApiKey, go to the "My projects" section in your personal account.'),
                        'name' => 'EWCRYPTOCLOUD_API_KEY',
                        'label' => $this->l('ApiKey'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l('To get an Secret Key, go to the "My projects" section in your personal account.'),
                        'name' => 'EWCRYPTOCLOUD_SECRET_KEY',
                        'label' => $this->l('Secret Key'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l('To get a shopId, you need to add a project in your personal account. The ID consists of 16 characters and is unique for each store.'),
                        'name' => 'EWCRYPTOCLOUD_SHOP_ID',
                        'label' => $this->l('ShopID'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Order state'),
                        'name' => 'EWCRYPTOCLOUD_ID_STATE_AFTER_CREATE',
                        'desc' => $this->l('Order state after create.'),
                        'options' => array(
                            'query' => OrderState::getOrderStates($this->context->language->id),
                            'name' => 'name',
                            'id' => 'id_order_state'
                        )
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Successful payment status'),
                        'name' => 'EWCRYPTOCLOUD_ID_STATE_SUCCESS',
                        'options' => array(
                            'query' => OrderState::getOrderStates($this->context->language->id),
                            'name' => 'name',
                            'id' => 'id_order_state'
                        )
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Status of failed payment'),
                        'name' => 'EWCRYPTOCLOUD_ID_STATE_FAIL',
                        'options' => array(
                            'query' => OrderState::getOrderStates($this->context->language->id),
                            'name' => 'name',
                            'id' => 'id_order_state'
                        )
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'EWCRYPTOCLOUD_API_KEY' => Configuration::get('EWCRYPTOCLOUD_API_KEY'),
            'EWCRYPTOCLOUD_SHOP_ID' => Configuration::get('EWCRYPTOCLOUD_SHOP_ID'),
            'EWCRYPTOCLOUD_ID_STATE_AFTER_CREATE' => Configuration::get('EWCRYPTOCLOUD_ID_STATE_AFTER_CREATE'),
            'EWCRYPTOCLOUD_ID_STATE_SUCCESS' => Configuration::get('EWCRYPTOCLOUD_ID_STATE_SUCCESS'),
            'EWCRYPTOCLOUD_ID_STATE_FAIL' => Configuration::get('EWCRYPTOCLOUD_ID_STATE_FAIL'),
            'EWCRYPTOCLOUD_SECRET_KEY' => Configuration::get('EWCRYPTOCLOUD_SECRET_KEY'),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('Crypto (BTC/LTC/ETH/USDT) - CryptoCloud'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
            ->setAdditionalInformation($this->l('Payment in cryptocurrency, you can pay even if you don\'t have it, the service will offer to buy cryptocurrency and pay for it.'));

        return [
            $option
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }
}
